/*
  RobotFace.cpp - Arduino Library for controlling RobotFace (MiRAE).
  Created by Casey Bennett, May 2013.
  www.CaseyBennett.com
  
   Copyright 2013 Casey Bennett

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License (Lesser GPL) as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "Arduino.h"
#include "RobotFace.h"
#include "MyTypes.h"


////////////////////////////////
//Constructor
///////////////

RobotFace::RobotFace(int lt_eye_pos, int rt_eye_pos, int lt_brow_pos, int rt_brow_pos, int lt_mc_pos, int rt_mc_pos, int lt_br_pos,
                  int rt_br_pos, int lt_mp_pos, int rt_mp_pos, int nt_pos, int np_pos,
                  int x_center_val, int x_min_val, int x_max_val, int y_center_val, int y_min_val, int y_max_val, 
                  int x_move_val, int y_move_val)
{
  //Set Iniital and Current Positions to passed in init values
  lt_eye_init = curr_et_l = lt_eye_pos;
  rt_eye_init = curr_et_r = rt_eye_pos;
  lt_brow_init = curr_bt_l = lt_brow_pos;
  rt_brow_init = curr_bt_r = rt_brow_pos;
  lt_mc_init = curr_mc_l = lt_mc_pos;
  rt_mc_init = curr_mc_r = rt_mc_pos;
  lt_br_init = curr_br_l = lt_br_pos;
  rt_br_init = curr_br_r = rt_br_pos;
  lt_mp_init = curr_mp_l = lt_mp_pos;
  rt_mp_init = curr_mp_r = rt_mp_pos;
  nt_init = curr_nt = nt_pos;
  np_init = curr_np = np_pos;
  
  //Facial Component Motion Degrees
  et_deg=1.0;
  bt_deg=1.0;
  mc_deg=1.0;
  br_deg=1.0;
  mp_deg=1.0;
  nt_deg=1.0;
  np_deg=1.0;
    
  //Camera Globals
  x_center = x_center_val;                         //Common values for these (on a 160x120 image, scale accordingly) are:
  x_min = x_min_val;                               //80, 20, 140, 55, 20, 90 (from x_center to y_max, in order)
  x_max = x_max_val;
  y_center = y_center_val;
  y_min = y_min_val;
  y_max = y_max_val;    
  //Move Globals
  x_move = x_move_val;                             //These values depend on how far you want the neck to move
  y_move = y_move_val;                             //For MiRAE, we typically set them around 27 and 15, respectively
  //Move Constraints
  nt_min=nt_init-y_move;
  nt_max=nt_init+y_move;
  np_min=np_init-x_move;
  np_max=np_init+x_move;
}


////////////////////////////////////////////////////////////////////////
//
// Useful Functions
//
///////////////////////////

void RobotFace::Initialize()
//Initializes servo positions
{ 
  Lt_Eye.write(lt_eye_init);  //Sets initial position
  Rt_Eye.write(rt_eye_init);  //Sets initial position
  Lt_Brow.write(lt_brow_init);  //Sets initial position
  Rt_Brow.write(rt_brow_init);  //Sets initial position
  Lt_MC.write(lt_mc_init);  //Sets initial position
  Rt_MC.write(rt_mc_init);  //Sets initial position
  Lt_BrowRaise.write(lt_br_init);  //Sets initial position
  Rt_BrowRaise.write(rt_br_init);  //Sets initial position
  Lt_MouthPart.write(lt_mp_init);  //Sets initial position
  Rt_MouthPart.write(rt_mp_init);  //Sets initial position
  Neck_Tilt.write(nt_init);  //Sets initial position
  Neck_Pan.write(np_init);  //Sets initial position
}


void RobotFace::Set_Degree(double et_degree, double bt_degree, double mc_degree, double br_degree, double mp_degree, 
                           double nt_degree, double np_degree)
/*Sets the degree of motion for each facial component individually, which is held as a 
  private variable in the RobotFace class.  Because the degree value is used when both making 
  and unmaking expressions, this function SHOULD ONLY BE CALLED when the robot is not 
  making any expression (i.e. neutral expression, aka servos in init position).*/
{
  et_deg = min(max(et_degree, 0.0),1.0);             //Ensure degree between 0-1
  bt_deg = min(max(bt_degree, 0.0),1.0);             //Ensure degree between 0-1
  mc_deg = min(max(mc_degree, 0.0),1.0);             //Ensure degree between 0-1
  br_deg = min(max(br_degree, 0.0),1.0);             //Ensure degree between 0-1
  mp_deg = min(max(mp_degree, 0.0),1.0);             //Ensure degree between 0-1
  nt_deg = min(max(nt_degree, 0.0),1.0);             //Ensure degree between 0-1
  np_deg = min(max(np_degree, 0.0),1.0);             //Ensure degree between 0-1
}

Returns RobotFace::Get_Curr_Pos()
/*Returns a MyTypes struct with the current position of
  each facial component*/
{
  Returns pos;
  pos.et_l=curr_et_l;
  pos.et_r=curr_et_r;
  pos.bt_l=curr_bt_l;
  pos.bt_r=curr_bt_r;
  pos.mc_l=curr_mc_l;
  pos.mc_r=curr_mc_r;
  pos.br_l=curr_br_l;
  pos.br_r=curr_br_r;
  pos.mp_l=curr_mp_l;
  pos.mp_r=curr_mp_r;
  pos.nt=curr_nt;
  pos.np=curr_np;
    
  return pos;  
}

void RobotFace::Neck_Move(double delta_nt, double delta_np)
/*Moves neck motors to new position, can be used to perform the "move" part of
  robot looking*/
{    
  int tot_steps= round(max(abs(delta_nt), abs(delta_np)));          //Step size is always max delta (want smooth gradated motion)
  double step_nt= double(delta_nt)/tot_steps;    //Step size for neck tilt
  double step_np= double(delta_np)/tot_steps;    //Step size for neck pan

  double old_nt =curr_nt;
  double old_np =curr_np;
  
  for(int t=0; t<tot_steps; t++)
  {
    //Neck Move
    double new_nt = old_nt + step_nt;
    double new_np = old_np + step_np;
    
    Neck_Tilt.write(round(new_nt));                   //Have to round off explicitly to avoid errors
    Neck_Pan.write(round(new_np));
  
    delay(15);
    old_nt =new_nt;
    old_np =new_np;
  }
  
  //Update current nt/np position values
  curr_nt =round(old_nt);                             //Have to round off explicitly to avoid errors
  curr_np =round(old_np);
}

void RobotFace::Robot_Look(long x, long y, int fixed_flag)
/*Uses x and y coordinates and grid system to have robot "look" at detected face.  
  Fixed Flag indicates whether camera is fixed (1) or not (0).*/
{
  double nt_new=0;
  double np_new=0;
    
  //Set tilt value 
  if(y<=y_center)
  {  nt_new= min(y_move, (y_move * double(y_center-y)/(y_center-y_min))); }
  else if(y>y_center)
  {  nt_new= max(-y_move, (-y_move * double(y-y_center)/(y_max-y_center))); }
  
  //Set pan value
  if(x<=x_center)
  {  np_new= max(-x_move, (-x_move * double(x_center-x)/(x_center-x_min))); }
  else if(x>x_center)
  {  np_new= min(x_move, (x_move * double(x-x_center)/(x_max-x_center))); }

  //Account for previous movement from center
  if(fixed_flag==1)
  {
    nt_new -= (curr_nt-nt_init);          
    np_new -= (curr_np-np_init);
  }
  else if(fixed_flag==0)
  { //(enforce min/max vals for nt and np)
    if((curr_nt+nt_new) < nt_min) 
    {  nt_new=nt_min-curr_nt;}
    if((curr_nt+nt_new) > nt_max) 
    {  nt_new=nt_max-curr_nt;}
    if((curr_np+np_new) < np_min) 
    {  np_new=np_min-curr_np;}
    if((curr_np+np_new) > np_max) 
    {  np_new=np_max-curr_np;}
  }
  else
  { exit(1); }                         //Throw error if fixed_flag is not set
  
  //Move
  Neck_Move(nt_new, np_new);
  
  //Test check
  Serial.println(curr_np);
  Serial.println(curr_nt);
}

void RobotFace::Look_Center()
/*Reset to "look" at center.*/
{
  double nt_new=nt_init-curr_nt;
  double np_new=np_init-curr_np;
  
  //Move
  Neck_Move(nt_new, np_new);
}

void RobotFace::Emerg_Center()
/*Emergency re-centering (e.g. if error). Doesn't use gradated motion like 
  the Look_Center() above, really just for cases when an error has occurred 
  and we are not sure where the head is currently looking anymore*/
{
  //Reset to center
  Neck_Tilt.write(nt_init);  //Sets initial position
  Neck_Pan.write(np_init);  //Sets initial position
  
  //Update current nt/np position values
  curr_nt = nt_init;
  curr_np = np_init;
}

void RobotFace::Eye_Saccade()
/*Random Eye Saccade movements around the current neck tilt and neck pan positions,
  i.e. the eye saccades are based on "muscle memory" of the last neck position"*/
{
  randomSeed(analogRead(0));
  int nt_sac=random(-2,3);              //Random eye saccade +/- 2 from center
  int np_sac=random(-2,3);              //Random eye saccade +/- 2 from center 
  Serial.println(np_sac);
  Serial.println(nt_sac);
  nt_sac=curr_nt+nt_sac;
  np_sac=curr_np+np_sac;
        
  //Move
  Neck_Tilt.write(nt_sac);
  Neck_Pan.write(np_sac);
  delay(20);  
}



///////////////////////////////////////////////////////////////////////////////////////////////////
//
// Facial Expression Functions
//
//////////////////////////////

//Sad Face
void RobotFace::Sad_Face(int dir, int neck_on)
/*Takes a direction (1=make expression, -1=unmake expression).
      Also takes flag indicating whether neck motion is turned on (=1) */
{ 
  int tot_steps=25;
  double delta_et=20 * et_deg;                    //Delta for eye tilt
  double delta_bt=15 * bt_deg;                    //Delta for brow tilt
  double delta_mc=50 * mc_deg;                    //Delta for mouth corners
  double delta_nt=10 * nt_deg;                    //Delta for neck tilt
  double step_et= delta_et/tot_steps * dir;       //Step size for eye tilt
  double step_bt= delta_bt/tot_steps * dir;       //Step size for brow tilt
  double step_mc= delta_mc/tot_steps * dir;       //Step size for mouth corners
  double step_nt= delta_nt/tot_steps * -dir;      //Step size for neck tilt

  double old_et_l=curr_et_l;
  double old_et_r=curr_et_r;
  double old_bt_l=curr_bt_l;
  double old_bt_r=curr_bt_r;
  double old_mc_l=curr_mc_l;
  double old_mc_r=curr_mc_r;
  double old_nt  =curr_nt;
  
  double new_nt=curr_nt;                                 //Need this in case neck_on=0
  
  for(int t=0; t<tot_steps; t++)
  {
    //Eye Tilt
    double new_et_l = old_et_l + step_et;
    double new_et_r = old_et_r - step_et;
    //Brow Tilt
    double new_bt_l = old_bt_l + step_bt;
    double new_bt_r = old_bt_r - step_bt;
    //Mouth Corners
    double new_mc_l = old_mc_l + step_mc;
    double new_mc_r = old_mc_r - step_mc;     
    //Neck Tilt
    if(neck_on==1)
    {new_nt = old_nt + step_nt;}
  
    Lt_Eye.write(round(new_et_l));                     //Have to round off explicitly to avoid errors
    Rt_Eye.write(round(new_et_r));
    Lt_Brow.write(round(new_bt_l)); 
    Rt_Brow.write(round(new_bt_r));
    Lt_MC.write(round(new_mc_l)); 
    Rt_MC.write(round(new_mc_r));
    Neck_Tilt.write(round(new_nt));
  
    delay(10);
    old_et_l=new_et_l;
    old_et_r=new_et_r;
    old_bt_l=new_bt_l;
    old_bt_r=new_bt_r;
    old_mc_l=new_mc_l;
    old_mc_r=new_mc_r;
    old_nt  =new_nt;
  }
  
  curr_et_l=round(old_et_l);                           //Have to round off explicitly to avoid errors
  curr_et_r=round(old_et_r);
  curr_bt_l=round(old_bt_l);
  curr_bt_r=round(old_bt_r);
  curr_mc_l=round(old_mc_l);
  curr_mc_r=round(old_mc_r);
  curr_nt=round(old_nt);
}
  
//Happy Face
void RobotFace::Happy_Face(int dir, int neck_on)
/*Takes a direction (1=make expression, -1=unmake expression).
      Also takes flag indicating whether neck motion is turned on (=1) */
{    
  int tot_steps=25;
  double delta_mc=50 * mc_deg;                    //Delta for mouth corners
  double delta_nt=2  * nt_deg;                    //Delta for neck tilt
  double step_mc= delta_mc/tot_steps * -dir;      //Step size for mouth corners
  double step_nt= delta_nt/tot_steps * dir;       //Step size for neck tilt

  double old_mc_l=curr_mc_l;
  double old_mc_r=curr_mc_r;
  double old_nt  =curr_nt;
  
  double new_nt=curr_nt;                             //Need this in case neck_on=0
  
  for(int t=0; t<tot_steps; t++)
  {
    //Mouth Corners
    double new_mc_l = old_mc_l + step_mc;
    double new_mc_r = old_mc_r - step_mc;
    //Neck Tilt
    if(neck_on==1)
    {new_nt = old_nt + step_nt;}
  
    Lt_MC.write(round(new_mc_l));                     //Have to round off explicitly to avoid errors
    Rt_MC.write(round(new_mc_r));
    Neck_Tilt.write(round(new_nt));
  
    delay(10);
    old_mc_l=new_mc_l;
    old_mc_r=new_mc_r;
    old_nt  =new_nt;
  }
  
  curr_mc_l=round(old_mc_l);                          //Have to round off explicitly to avoid errors
  curr_mc_r=round(old_mc_r);
  curr_nt=round(old_nt);
}
  
//Angry Face
void RobotFace::Angry_Face(int dir, int neck_on)
/*Takes a direction (1=make expression, -1=unmake expression).
      Also takes flag indicating whether neck motion is turned on (=1) */
{ 
  int tot_steps=25;
  double delta_et=20 * et_deg;                    //Delta for eye tilt
  double delta_bt=40 * bt_deg;                    //Delta for brow tilt
  double delta_nt=2  * nt_deg;                    //Delta for neck tilt
  double step_et= delta_et/tot_steps * -dir;      //Step size for eye tilt
  double step_bt= delta_bt/tot_steps * -dir;      //Step size for brow tilt
  double step_nt= delta_nt/tot_steps * -dir;      //Step size for neck tilt

  double old_et_l=curr_et_l;
  double old_et_r=curr_et_r;
  double old_bt_l=curr_bt_l;
  double old_bt_r=curr_bt_r;
  double old_nt  =curr_nt;
  
  double new_nt=curr_nt;                                 //Need this in case neck_on=0
  
  for(int t=0; t<tot_steps; t++)
  {
    //Eye Tilt
    double new_et_l = old_et_l + step_et;
    double new_et_r = old_et_r - step_et;
    //Brow Tilt
    double new_bt_l = old_bt_l + step_bt;
    double new_bt_r = old_bt_r - step_bt;
    //Neck Tilt
    if(neck_on==1)
    {new_nt = old_nt + step_nt;}
  
    Lt_Eye.write(round(new_et_l));                     //Have to round off explicitly to avoid errors
    Rt_Eye.write(round(new_et_r));
    Lt_Brow.write(round(new_bt_l)); 
    Rt_Brow.write(round(new_bt_r));
    Neck_Tilt.write(round(new_nt));
  
    delay(10);
    old_et_l=new_et_l;
    old_et_r=new_et_r;
    old_bt_l=new_bt_l;
    old_bt_r=new_bt_r;
    old_nt  =new_nt;
  }
  
  curr_et_l=round(old_et_l);                           //Have to round off explicitly to avoid errors
  curr_et_r=round(old_et_r);
  curr_bt_l=round(old_bt_l);
  curr_bt_r=round(old_bt_r);
  curr_nt=round(old_nt);
}
  
//Fear Face
void RobotFace::Fear_Face(int dir, int neck_on)
/*Takes a direction (1=make expression, -1=unmake expression).
      Also takes flag indicating whether neck motion is turned on (=1) */
{ 
  int tot_steps=25;
  double delta_bt=30 * bt_deg;                    //Delta for brow tilt
  double delta_mc=20 * mc_deg;                    //Delta for mouth corners
  double delta_mp=10 * mp_deg;                    //Delta for mouth part
  double delta_nt=5  * nt_deg;                    //Delta for neck tilt
  double step_bt= delta_bt/tot_steps * dir;       //Step size for brow tilt
  double step_mc= delta_mc/tot_steps * dir;       //Step size for mouth corners
  double step_mp= delta_mp/tot_steps * dir;       //Step size for mouth part
  double step_nt= delta_nt/tot_steps * dir;       //Step size for neck tilt

  double old_bt_l=curr_bt_l;
  double old_bt_r=curr_bt_r;
  double old_mc_l=curr_mc_l;
  double old_mc_r=curr_mc_r;
  double old_mp_l=curr_mp_l;
  double old_mp_r=curr_mp_r;
  double old_nt  =curr_nt;
  
  double new_nt=curr_nt;                                 //Need this in case neck_on=0
  
  for(int t=0; t<tot_steps; t++)
  {
    //Brow Tilt
    double new_bt_l = old_bt_l + step_bt;
    double new_bt_r = old_bt_r - step_bt;
    //Mouth Corners
    double new_mc_l = old_mc_l + step_mc;
    double new_mc_r = old_mc_r - step_mc;
    //Mouth Part
    double new_mp_l = old_mp_l + step_mp;
    double new_mp_r = old_mp_r - step_mp;
    //Neck Tilt
    if(neck_on==1)
    {new_nt = old_nt + step_nt;}
    
    Lt_Brow.write(round(new_bt_l));                     //Have to round off explicitly to avoid errors 
    Rt_Brow.write(round(new_bt_r));
    Lt_MC.write(round(new_mc_l)); 
    Rt_MC.write(round(new_mc_r));
    Lt_MouthPart.write(round(new_mp_l)); 
    Rt_MouthPart.write(round(new_mp_r));
    Neck_Tilt.write(round(new_nt));
   
    delay(10);
    old_bt_l=new_bt_l;
    old_bt_r=new_bt_r;
    old_mc_l=new_mc_l;
    old_mc_r=new_mc_r;
    old_mp_l=new_mp_l;
    old_mp_r=new_mp_r;
    old_nt  =new_nt;
  }
  
  curr_bt_l=round(old_bt_l);                            //Have to round off explicitly to avoid errors
  curr_bt_r=round(old_bt_r);
  curr_mc_l=round(old_mc_l);
  curr_mc_r=round(old_mc_r);
  curr_mp_l=round(old_mp_l);
  curr_mp_r=round(old_mp_r);
  curr_nt=round(old_nt);
}
  
//Surprise Face
void RobotFace::Surprise_Face(int dir, int neck_on)
/*Takes a direction (1=make expression, -1=unmake expression).
      Also takes flag indicating whether neck motion is turned on (=1) */
{ 
  int tot_steps=25;
  double delta_br=40 * br_deg;                    //Delta for brow raise
  double delta_mp=30 * mp_deg;                    //Delta for mouth part
  double delta_nt=6  * nt_deg;                    //Delta for neck tilt
  double step_br= delta_br/tot_steps * dir;       //Step size for brow raise
  double step_mp= delta_mp/tot_steps * dir;       //Step size for mouth part
  double step_nt= delta_nt/tot_steps * dir;       //Step size for neck tilt

  double old_br_l=curr_br_l;
  double old_br_r=curr_br_r;
  double old_mp_l=curr_mp_l;
  double old_mp_r=curr_mp_r;
  double old_nt  =curr_nt;
  
  double new_nt=curr_nt;                                 //Need this in case neck_on=0
  
  for(int t=0; t<tot_steps; t++)
  {
    //Brow Raise
    double new_br_l = old_br_l + step_br;
    double new_br_r = old_br_r - step_br;
    //Mouth Part
    double new_mp_l = old_mp_l + step_mp;
    double new_mp_r = old_mp_r - step_mp;
    //Neck Tilt
    if(neck_on==1)
    {new_nt = old_nt + step_nt;}
  
    Lt_BrowRaise.write(round(new_br_l));                     //Have to round off explicitly to avoid errors
    Rt_BrowRaise.write(round(new_br_r));
    Lt_MouthPart.write(round(new_mp_l)); 
    Rt_MouthPart.write(round(new_mp_r));
    Neck_Tilt.write(round(new_nt));
  
    delay(10);
    old_br_l=new_br_l;
    old_br_r=new_br_r;
    old_mp_l=new_mp_l;
    old_mp_r=new_mp_r;
    old_nt  =new_nt;
  }
  
  curr_br_l=round(old_br_l);                                 //Have to round off explicitly to avoid errors
  curr_br_r=round(old_br_r);
  curr_mp_l=round(old_mp_l);
  curr_mp_r=round(old_mp_r);
  curr_nt=round(old_nt);
}


//Interest Face
void RobotFace::Interest_Face(int dir, int neck_on)
/*This is a hack for entertaining children, makes a quizzical eyebrow look.  Note that unlike 
  the other expressions, this one is asymmetrical (hence the separate step sizes for right
  and left facial features).  Takes a direction (1=make expression, -1=unmake expression).
  Also takes flag indicating whether neck motion is turned on (=1) */
{ 
  int tot_steps=20;
  double delta_br=40 * br_deg;                    //Delta for brow raise
  double delta_bt=10 * bt_deg;                    //Delta for brow tilt
  double step_br_r= delta_br/tot_steps * dir;       //Step size for brow raise right
  double step_br_l= delta_br/tot_steps * dir;       //Step size for brow raise left
  double step_bt_r= delta_bt/tot_steps * dir;       //Step size for neck tilt right

  double old_br_l=curr_br_l;
  double old_br_r=curr_br_r;
  double old_bt_l=curr_bt_l;
  double old_bt_r=curr_bt_r;
  
  for(int t=0; t<tot_steps; t++)
  {
    //Brow Raise
    double new_br_r = old_br_r - step_br_r;
    double new_br_l = old_br_l + step_br_l;
    //Brow Tilt
    double new_bt_r = old_bt_r + step_bt_r;
  
    Rt_BrowRaise.write(round(new_br_r));                     //Have to round off explicitly to avoid errors
    Lt_BrowRaise.write(round(new_br_l));
    Rt_Brow.write(round(new_bt_r));
  
    delay(10);
    old_br_r=new_br_r;
    old_br_l=new_br_l;
    old_bt_r=new_bt_r;
  }
  
  curr_br_r=round(old_br_r);                                 //Have to round off explicitly to avoid errors
  curr_br_l=round(old_br_l);
  curr_bt_r=round(old_bt_r);
}


void RobotFace::Make_Face(int face_choice, int delay_time, int neck_on)
//This is used in special circumstances for quickly making and unmaking a facial expression, based on
//  passed in face_choice variable and delay_time (in milliseconds).  Neck motion may be turned on or off.
{
  Returns return1;
  
  if(face_choice==1)
  {  
    //Sad Face
    Sad_Face(1, neck_on);
    delay(delay_time);
    //De-sad
    Sad_Face(-1, neck_on);
    delay(delay_time);
  }
  else if(face_choice==2)
  {
    //Happy Face
    Happy_Face(1, neck_on);
    delay(delay_time);
    //De-happy
    Happy_Face(-1, neck_on);
    delay(delay_time);
  }
  else if(face_choice==3)
  {
    //Angry Face
    Angry_Face(1, neck_on);
    delay(delay_time);
    //De-angry
    Angry_Face(-1, neck_on);
    delay(delay_time);
  }
  else if (face_choice==4)
  {
    //Fear Face
    Fear_Face(1, neck_on);
    delay(delay_time);
    //De-Fear
    Fear_Face(-1, neck_on);
    delay(delay_time);
  }
  else if (face_choice==5)
  {
    //Surprise Face
    Surprise_Face(1, neck_on);
    delay(delay_time);
    //De-surprise
    Surprise_Face(-1, neck_on);
    delay(delay_time);
  }
}

////////////////////////////////////////////////////////////////////////
//
// Communication Functions
//
///////////////////////////

long RobotFace::getSerial()
/*For receiving USB communication through an arduino from an external computer.  
  Reads serial integer values (single or multiple digits) and returns the value as 
  an unsigned long integer.  Currently written to only work with positive integers.
  Sending a backslash ('/') is used as a termination signal (i.e. end of number)*/
{
  long serialdata = 0;
  int inbyte = 0;                                           // for incoming serial data

  while (inbyte != '/')                                     //Uses backslash as termination signal
  {
    inbyte = Serial.read();  
    if (inbyte > 0 && inbyte != '/')                        
    { 
      serialdata = serialdata * 10 + inbyte - '0';
      //Serial.println(serialdata);
    }
  }  
  return serialdata;  
}
